@@ -15,33 +15,100 @@
 from sklearn.svm import LinearSVC
 # from b4msa.textmodel import TextModel
 import numpy as np
-from b4msa.utils import read_data_labels, read_data
-from gensim.matutils import corpus2csc
+from b4msa.utils import read_data_labels, read_data, tweet_iterator
 from sklearn import preprocessing
-from sklearn import cross_validation
+from sklearn.model_selection import StratifiedKFold
 from b4msa.textmodel import TextModel
 from multiprocessing import Pool
+from scipy.sparse import csr_matrix
 import logging
 logging.basicConfig(format='%(asctime)s : %(levelname)s :%(message)s')
 
 
 class SVC(object):
-    def __init__(self, model):
-        self.svc = LinearSVC()
+    """Classifier
+
+    :param model: TextModel
+    :type model: class
+
+    Usage:
+
+    >>> from b4msa.textmodel import TextModel
+    >>> from b4msa.classifier import SVC
+    >>> corpus = ['buenos dias', 'catedras conacyt', 'categorizacion de texto ingeotec']
+    >>> textmodel = TextModel(corpus)
+    >>> svc = SVC(textmodel)
+    >>> _ = svc.fit([textmodel[x] for x in corpus], [1, 0, 0])
+    >>> svc.predict_text('hola')
+    0
+    """
+    def __init__(self, model, **kwargs):
+        self.svc = LinearSVC(**kwargs)
         self.model = model
-        self.num_terms = -1
+
+    @property
+    def num_terms(self):
+        """Dimension which is the number of terms of the corpus
+
+        :rtype: int
+        """
+
+        try:
+            return self._num_terms
+        except AttributeError:
+            self._num_terms = None
+        return None
+
+    def tonp(self, X):
+        """Sparse representation to sparce matrix
+
+        :param X: Sparse representation of matrix
+        :type X: list
+        :rtype: csr_matrix
+        """
+
+        data = []
+        row = []
+        col = []
+        for r, x in enumerate(X):
+            cc = [_[0] for _ in x if np.isfinite(_[1]) and (self.num_terms is None or _[0] < self.num_terms)]
+            col += cc
+            data += [_[1] for _ in x if np.isfinite(_[1]) and (self.num_terms is None or _[0] < self.num_terms)]
+            _ = [r] * len(cc)
+            row += _
+        if self.num_terms is None:
+            _ = csr_matrix((data, (row, col)))
+            self._num_terms = _.shape[1]
+            return _
+        return csr_matrix((data, (row, col)), shape=(len(X), self.num_terms))
 
     def fit(self, X, y):
-        X = corpus2csc(X).T
-        self.num_terms = X.shape[1]
+        """Train the classifier
+
+        :param X: inputs - independent variables
+        :type X: lst
+        :param y: output - dependent variable
+
+        :rtype: instance
+        """
+
+        X = self.tonp(X)
         self.le = preprocessing.LabelEncoder()
         self.le.fit(y)
         y = self.le.transform(y)
+        if self.num_terms == 0:
+            return self
         self.svc.fit(X, y)
         return self
 
+    def decision_function(self, Xnew):
+        Xnew = self.tonp(Xnew)
+        return self.svc.decision_function(Xnew)
+
     def predict(self, Xnew):
-        Xnew = corpus2csc(Xnew, num_terms=self.num_terms).T
+        if self.num_terms == 0:
+            return self.le.inverse_transform(np.zeros(len(Xnew), dtype=np.int))
+        Xnew = self.tonp(Xnew)
         ynew = self.svc.predict(Xnew)
         return self.le.inverse_transform(ynew)
 
@@ -64,9 +131,7 @@ class SVC(object):
 
     @classmethod
     def predict_kfold(cls, X, y, n_folds=10, seed=0, textModel_params={},
-                      kfolds=None,
-                      pool=None,
-                      use_tqdm=True):
+                      kfolds=None, pool=None, use_tqdm=True):
         try:
             from tqdm import tqdm
         except ImportError:
@@ -77,16 +142,13 @@ class SVC(object):
         y = np.array(le.transform(y))
         hy = np.zeros(len(y), dtype=np.int)
         if kfolds is None:
-            kfolds = cross_validation.StratifiedKFold(y,
-                                                      n_folds=n_folds,
-                                                      shuffle=True,
-                                                      random_state=seed)
+            kfolds = StratifiedKFold(n_splits=n_folds, shuffle=True,
+                                     random_state=seed).split(X, y)
         args = [(X, y, tr, ts, textModel_params) for tr, ts in kfolds]
         if pool is not None:
             if use_tqdm:
                 res = [x for x in tqdm(pool.imap_unordered(cls.train_predict_pool, args),
-                                       desc='Params',
-                                       total=len(args))]
+                                       desc='Params', total=len(args))]
             else:
                 res = [x for x in pool.imap_unordered(cls.train_predict_pool, args)]
         else:
@@ -100,6 +162,8 @@ class SVC(object):
     @classmethod
     def train_predict_pool(cls, args):
         X, y, tr, ts, textModel_params = args
+        params = TextModel.params()
+        textModel_params = {k: v for k, v in textModel_params.items() if k in params}
         t = TextModel([X[x] for x in tr], **textModel_params)
         m = cls(t).fit([t[X[x]] for x in tr], [y[x] for x in tr])
         return ts, np.array(m.predict([t[X[x]] for x in ts]))
@@ -124,7 +188,9 @@ class SVC(object):
 
     @classmethod
     def fit_from_file(cls, fname, textModel_params={}):
-        X, y = read_data_labels(fname)
-        model = TextModel(X, **textModel_params)
+        D = [x for x in tweet_iterator(fname)]
+        # X, y = read_data_labels(fname)
+        y = [x['klass'] for x in D]
+        model = TextModel(D, **textModel_params)
         svc = cls(model)
-        return svc.fit([model[x] for x in X], y)
+        return svc.fit([model[x] for x in D], y)
