@@ -6,7 +6,7 @@ import logging
 from time import time
 from sklearn.metrics import f1_score, accuracy_score, recall_score, precision_score
 from sklearn import preprocessing
-from sklearn import cross_validation
+from sklearn.model_selection import StratifiedKFold
 
 try:
     from tqdm import tqdm
@@ -24,24 +24,26 @@ OPTION_DELETE = 'delete'
 BASIC_OPTIONS = [OPTION_DELETE, OPTION_GROUP, OPTION_NONE]
 
 _BASE_PARAMS = dict(
-    strip_diac=[False, True],
+    del_diac=[False, True],
     num_option=BASIC_OPTIONS,
     usr_option=BASIC_OPTIONS,
     url_option=BASIC_OPTIONS,
     emo_option=BASIC_OPTIONS,
     lc=[False, True],
-    del_dup1=[False, True],
+    del_dup=[False, True],
+    # knowledge=[False, True],
     token_list=[-2, -1, 1, 2, 3, 4, 5, 6, 7],
 )
 
 _BASE_PARAMS_LANG = dict(
-    strip_diac=[False, True],
+    del_diac=[False, True],
     num_option=BASIC_OPTIONS,
     usr_option=BASIC_OPTIONS,
     url_option=BASIC_OPTIONS,
     emo_option=BASIC_OPTIONS,
     lc=[False, True],
-    del_dup1=[False, True],
+    del_dup=[False, True],
+    # knowledge=[False, True],
     token_list=[-2, -1, 1, 2, 3, 4, 5, 6, 7],
     negation=[False, True],
     stemming=[False, True],
@@ -175,14 +177,9 @@ class Wrapper(object):
         self.cls = cls
         self.pool = pool
         np.random.seed(seed)
-        self.kfolds = [
-            x for x in cross_validation.StratifiedKFold(
-                y,
-                n_folds=n_folds,
-                shuffle=True,
-                random_state=seed
-            )
-        ]
+        self.kfolds = [x for x in StratifiedKFold(n_splits=n_folds, shuffle=True,
+                                                  random_state=seed).split(np.zeros(self.y.shape[0]),
+                                                                           self.y)]
 
     def f(self, conf_code):
         conf, code = conf_code
@@ -192,23 +189,35 @@ class Wrapper(object):
                                     kfolds=self.kfolds,
                                     pool=self.pool,
                                     use_tqdm=False)
-
-        conf['_macrof1'] = f1_score(self.y, hy, average='macro')
-        conf['_all_f1'] = M = {self.le.inverse_transform([klass])[0]: f1 for klass, f1 in enumerate(f1_score(self.y, hy, average=None))}
-        conf['_all_recall'] = M = {self.le.inverse_transform([klass])[0]: f1 for klass, f1 in enumerate(recall_score(self.y, hy, average=None))}
-        conf['_all_precision'] = M = {self.le.inverse_transform([klass])[0]: f1 for klass, f1 in enumerate(precision_score(self.y, hy, average=None))}
-        conf['_microf1'] = f1_score(self.y, hy, average='micro')
-        conf['_weightedf1'] = f1_score(self.y, hy, average='weighted')
-        conf['_accuracy'] = accuracy_score(self.y, hy)
-        if self.score.startswith('avgf1'):
-            _, k1, k2 = self.score.split(':')
-            conf['_' + self.score] = (M[k1] + M[k2]) / 2
-
-        conf['_score'] = conf['_' + self.score]
+        self.compute_score(conf, hy)
         conf['_time'] = (time() - st) / self.n_folds
         return conf
 
-                
+    def compute_score(self, conf, hy):
+        RS = recall_score(self.y, hy, average=None)
+        conf['_all_f1'] = M = {str(self.le.inverse_transform([klass])[0]): f1 for klass, f1 in enumerate(f1_score(self.y, hy, average=None))}
+        conf['_all_recall'] = {str(self.le.inverse_transform([klass])[0]): f1 for klass, f1 in enumerate(RS)}
+        conf['_all_precision'] = N = {str(self.le.inverse_transform([klass])[0]): f1 for klass, f1 in enumerate(precision_score(self.y, hy, average=None))}
+        conf['_macrorecall'] = np.mean(RS)
+        if len(self.le.classes_) == 2:
+            conf['_macrof1'] = np.mean(np.array([v for v in conf['_all_f1'].values()]))
+            conf['_weightedf1'] = conf['_microf1'] = f1_score(self.y, hy, average='binary')
+        else:
+            conf['_macrof1'] = f1_score(self.y, hy, average='macro')
+            conf['_microf1'] = f1_score(self.y, hy, average='micro')
+            conf['_weightedf1'] = f1_score(self.y, hy, average='weighted')
+        conf['_accuracy'] = accuracy_score(self.y, hy)
+        if self.score.startswith('avgf1:'):
+            _, k1, k2 = self.score.split(':')
+            conf['_' + self.score] = (M[k1] + M[k2]) / 2
+        elif self.score.startswith('avgf1f0:'):
+            _, k1, k2 = self.score.split(':')
+            pos = (M[k1] + N[k1]) / 2.
+            neg = (M[k2] + N[k2]) / 2.
+            conf['_' + self.score] = (pos + neg) / 2.
+        conf['_score'] = conf['_' + self.score]
+
+
 def get_filename(kwargs, basename=None):
     L = []
     if basename:
